@@ -1,7 +1,7 @@
 use std::collections::HashMap;
 
 use chrono::Utc;
-use eyre::{eyre, Result};
+use eyre::{bail, Result};
 use reqwest::header::{HeaderMap, AUTHORIZATION, USER_AGENT};
 use reqwest::{StatusCode, Url};
 use sodiumoxide::crypto::secretbox;
@@ -31,17 +31,17 @@ pub async fn register(
     username: &str,
     email: &str,
     password: &str,
-) -> Result<RegisterResponse<'static>> {
+) -> Result<RegisterResponse> {
     let mut map = HashMap::new();
     map.insert("username", username);
     map.insert("email", email);
     map.insert("password", password);
 
     let url = format!("{}/user/{}", address, username);
-    let resp = reqwest::blocking::get(url)?;
+    let resp = reqwest::get(url).await?;
 
     if resp.status().is_success() {
-        return Err(eyre!("username already in use"));
+        bail!("username already in use");
     }
 
     let url = format!("{}/register", address);
@@ -54,14 +54,14 @@ pub async fn register(
         .await?;
 
     if !resp.status().is_success() {
-        return Err(eyre!("failed to register user"));
+        bail!("failed to register user");
     }
 
     let session = resp.json::<RegisterResponse>().await?;
     Ok(session)
 }
 
-pub async fn login(address: &str, req: LoginRequest<'_>) -> Result<LoginResponse<'static>> {
+pub async fn login(address: &str, req: LoginRequest) -> Result<LoginResponse> {
     let url = format!("{}/login", address);
     let client = reqwest::Client::new();
 
@@ -73,7 +73,7 @@ pub async fn login(address: &str, req: LoginRequest<'_>) -> Result<LoginResponse
         .await?;
 
     if resp.status() != reqwest::StatusCode::OK {
-        return Err(eyre!("invalid login details"));
+        bail!("invalid login details");
     }
 
     let session = resp.json::<LoginResponse>().await?;
@@ -102,7 +102,7 @@ impl<'a> Client<'a> {
         let resp = self.client.get(url).send().await?;
 
         if resp.status() != StatusCode::OK {
-            return Err(eyre!("failed to get count (are you logged in?)"));
+            bail!("failed to get count (are you logged in?)");
         }
 
         let count = resp.json::<CountResponse>().await?;
@@ -142,7 +142,7 @@ impl<'a> Client<'a> {
         Ok(history)
     }
 
-    pub async fn post_history(&self, history: &[AddHistoryRequest<'_, String>]) -> Result<()> {
+    pub async fn post_history(&self, history: &[AddHistoryRequest]) -> Result<()> {
         let url = format!("{}/history", self.sync_addr);
         let url = Url::parse(url.as_str())?;
 
