@@ -8,11 +8,12 @@
 // clients must share the secret in order to be able to sync, as it is needed
 // to decrypt
 
-use std::fs::File;
+use fs_err as fs;
+use serde::{Deserialize, Serialize};
 use std::io::prelude::*;
 use std::path::PathBuf;
 
-use eyre::{eyre, Result};
+use eyre::{eyre, Context, Result};
 use sodiumoxide::crypto::secretbox;
 
 use crate::history::History;
@@ -30,7 +31,7 @@ pub fn new_key(settings: &Settings) -> Result<secretbox::Key> {
     let key = secretbox::gen_key();
     let encoded = encode_key(key.clone())?;
 
-    let mut file = File::create(path)?;
+    let mut file = fs::File::create(path)?;
     file.write_all(encoded.as_bytes())?;
 
     Ok(key)
@@ -41,7 +42,7 @@ pub fn load_key(settings: &Settings) -> Result<secretbox::Key> {
     let path = settings.key_path.as_str();
 
     let key = if PathBuf::from(path).exists() {
-        let key = std::fs::read_to_string(path)?;
+        let key = fs_err::read_to_string(path)?;
         decode_key(key)?
     } else {
         new_key(settings)?
@@ -54,13 +55,13 @@ pub fn load_encoded_key(settings: &Settings) -> Result<String> {
     let path = settings.key_path.as_str();
 
     if PathBuf::from(path).exists() {
-        let key = std::fs::read_to_string(path)?;
+        let key = fs::read_to_string(path)?;
         Ok(key)
     } else {
         let key = secretbox::gen_key();
         let encoded = encode_key(key)?;
 
-        let mut file = File::create(path)?;
+        let mut file = fs::File::create(path)?;
         file.write_all(encoded.as_bytes())?;
 
         Ok(encoded)
@@ -68,15 +69,16 @@ pub fn load_encoded_key(settings: &Settings) -> Result<String> {
 }
 
 pub fn encode_key(key: secretbox::Key) -> Result<String> {
-    let buf = rmp_serde::to_vec(&key)?;
+    let buf = rmp_serde::to_vec(&key).wrap_err("could not encode key to message pack")?;
     let buf = base64::encode(buf);
 
     Ok(buf)
 }
 
 pub fn decode_key(key: String) -> Result<secretbox::Key> {
-    let buf = base64::decode(key)?;
-    let buf: secretbox::Key = rmp_serde::from_read_ref(&buf)?;
+    let buf = base64::decode(key).wrap_err("encryption key is not a valid base64 encoding")?;
+    let buf: secretbox::Key = rmp_serde::from_slice(&buf)
+        .wrap_err("encryption key is not a valid message pack encoding")?;
 
     Ok(buf)
 }
@@ -96,7 +98,7 @@ pub fn decrypt(encrypted_history: &EncryptedHistory, key: &secretbox::Key) -> Re
     let plaintext = secretbox::open(&encrypted_history.ciphertext, &encrypted_history.nonce, key)
         .map_err(|_| eyre!("failed to open secretbox - invalid key?"))?;
 
-    let history = rmp_serde::from_read_ref(&plaintext)?;
+    let history = rmp_serde::from_slice(&plaintext)?;
 
     Ok(history)
 }
@@ -133,14 +135,11 @@ mod test {
         // test decryption works
         // this should pass
         match decrypt(&e1, &key1) {
-            Err(e) => assert!(false, "failed to decrypt, got {}", e),
+            Err(e) => panic!("failed to decrypt, got {}", e),
             Ok(h) => assert_eq!(h, history),
         };
 
         // this should err
-        match decrypt(&e2, &key1) {
-            Ok(_) => assert!(false, "expected an error decrypting with invalid key"),
-            Err(_) => {}
-        };
+        let _ = decrypt(&e2, &key1).expect_err("expected an error decrypting with invalid key");
     }
 }
