@@ -1,4 +1,5 @@
-use std::fs::{create_dir_all, File};
+use fs_err::{create_dir_all, File};
+use serde::Deserialize;
 use std::io::prelude::*;
 use std::path::{Path, PathBuf};
 
@@ -22,6 +23,21 @@ pub enum SearchMode {
     Fuzzy,
 }
 
+#[derive(Clone, Debug, Deserialize, Copy)]
+pub enum FilterMode {
+    #[serde(rename = "global")]
+    Global,
+
+    #[serde(rename = "host")]
+    Host,
+
+    #[serde(rename = "session")]
+    Session,
+
+    #[serde(rename = "directory")]
+    Directory,
+}
+
 // FIXME: Can use upstream Dialect enum if https://github.com/stevedonovan/chrono-english/pull/16 is merged
 #[derive(Clone, Debug, Deserialize, Copy)]
 pub enum Dialect {
@@ -64,6 +80,7 @@ pub struct Settings {
     pub key_path: String,
     pub session_path: String,
     pub search_mode: SearchMode,
+    pub filter_mode: FilterMode,
     // This is automatically loaded when settings is created. Do not set in
     // config! Keep secrets and settings apart.
     pub session_token: String,
@@ -76,7 +93,7 @@ impl Settings {
 
         let sync_time_path = data_dir.join("last_sync_time");
 
-        std::fs::write(sync_time_path, Utc::now().to_rfc3339())?;
+        fs_err::write(sync_time_path, Utc::now().to_rfc3339())?;
 
         Ok(())
     }
@@ -91,7 +108,7 @@ impl Settings {
             return Ok(Utc.ymd(1970, 1, 1).and_hms(0, 0, 0));
         }
 
-        let time = std::fs::read_to_string(sync_time_path)?;
+        let time = fs_err::read_to_string(sync_time_path)?;
         let time = chrono::DateTime::parse_from_rfc3339(time.as_str())?;
 
         Ok(time.with_timezone(&Utc))
@@ -146,9 +163,14 @@ impl Settings {
             .set_default("sync_frequency", "1h")?
             .set_default("sync_address", "https://api.atuin.sh")?
             .set_default("search_mode", "prefix")?
+            .set_default("filter_mode", "global")?
             .set_default("session_token", "")?
             .set_default("style", "auto")?
-            .add_source(Environment::with_prefix("atuin").separator("_"));
+            .add_source(
+                Environment::with_prefix("atuin")
+                    .prefix_separator("_")
+                    .separator("__"),
+            );
 
         config_builder = if config_file.exists() {
             config_builder.add_source(ConfigFile::new(
@@ -184,7 +206,7 @@ impl Settings {
 
         // Finally, set the auth token
         if Path::new(session_path.to_string().as_str()).exists() {
-            let token = std::fs::read_to_string(session_path.to_string())?;
+            let token = fs_err::read_to_string(session_path.to_string())?;
             settings.session_token = token.trim().to_string();
         } else {
             settings.session_token = String::from("not logged in");
