@@ -1,4 +1,5 @@
-use std::fs::{create_dir_all, File};
+use fs_err::{create_dir_all, File};
+use serde::{Deserialize, Serialize};
 use std::io::prelude::*;
 use std::path::PathBuf;
 
@@ -37,8 +38,11 @@ impl Settings {
             .set_default("host", "127.0.0.1")?
             .set_default("port", 8888)?
             .set_default("open_registration", false)?
-            .set_default("db_uri", "default_uri")?
-            .add_source(Environment::with_prefix("atuin").separator("_"));
+            .add_source(
+                Environment::with_prefix("atuin")
+                    .prefix_separator("_")
+                    .separator("__"),
+            );
 
         config_builder = if config_file.exists() {
             config_builder.add_source(ConfigFile::new(
