@@ -1,5 +1,7 @@
+use tracing_subscriber::{fmt, prelude::*, EnvFilter};
+
 use clap::Parser;
-use eyre::Result;
+use eyre::{Context, Result};
 
 use atuin_server::launch;
 use atuin_server::settings::Settings;
@@ -10,7 +12,7 @@ pub enum Cmd {
     /// Start the server
     Start {
         /// The host address to bind
-        #[clap(long, short)]
+        #[clap(long)]
         host: Option<String>,
 
         /// The port to bind
@@ -20,7 +22,14 @@ pub enum Cmd {
 }
 
 impl Cmd {
-    pub async fn run(&self, settings: &Settings) -> Result<()> {
+    pub async fn run(self) -> Result<()> {
+        tracing_subscriber::registry()
+            .with(fmt::layer())
+            .with(EnvFilter::from_default_env())
+            .init();
+
+        let settings = Settings::new().wrap_err("could not load server settings")?;
+
         match self {
             Self::Start { host, port } => {
                 let host = host
